# Generated by Django 2.2 on 2019-05-28 02:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0006_auto_20190528_0258'),
        ('engines', '0052_cluster_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='scale',
            name='stack',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.PROTECT, to='engines.Stack'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='stack',
            name='engines',
            field=models.ManyToManyField(to='engines.Engine'),
        ),
        migrations.AlterField(
            model_name='engine',
            name='name',
            field=models.CharField(max_length=50, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name='engine',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='engine',
            name='components',
        ),
        migrations.RemoveField(
            model_name='engine',
            name='stack',
        ),
        migrations.DeleteModel(
            name='Component',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-14 07:31

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0031_auto_20190413_1111'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cluster',
            name='current_step',
        ),
        migrations.RemoveField(
            model_name='cluster',
            name='instances',
        ),
        migrations.RemoveField(
            model_name='cluster',
            name='remedy_script_todo',
        ),
        migrations.RemoveField(
            model_name='cluster',
            name='step_instances',
        ),
        migrations.RemoveField(
            model_name='step',
            name='created_time',
        ),
        migrations.RemoveField(
            model_name='step',
            name='remedy_script_todo',
        ),
        migrations.AddField(
            model_name='step',
            name='status',
            field=models.PositiveIntegerField(choices=[(0, 'null'), (1, 'active'), (2, 'block'), (3, 'suspend'), (4, 'shutdown'), (5, 'poweroff'), (6, 'breakdown'), (7, 'pause'), (8, 'failure'), (9, 'host_lost'), (10, 'instance_lost'), (11, 'building')], default=11, editable=False),
        ),
        migrations.AddField(
            model_name='step',
            name='uuid',
            field=models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False),
        ),
    ]



# Generated by Django 2.2 on 2019-04-28 22:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0038_auto_20190427_0841'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cloud',
            name='_driver',
            field=models.CharField(choices=[('clouds.drivers.evcloud', 'clouds.drivers.evcloud'), ('clouds.drivers.openstack', 'clouds.drivers.openstack')], max_length=50),
        ),
    ]



# Generated by Django 2.2 on 2019-05-27 21:35

from django.conf import settings
from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('engines', '0052_cluster_active'),
        ('data', '0004_auto_20190527_2123'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dataset',
            name='uri',
        ),
        migrations.AlterUniqueTogether(
            name='datainstance',
            unique_together={('engine', 'name'), ('name', 'cluster', 'owner')},
        ),
        migrations.RemoveField(
            model_name='datainstance',
            name='uri_suffix',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-05 03:29

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0003_auto_20190403_0352'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='scale',
            name='remedy_script',
        ),
        migrations.AddField(
            model_name='component',
            name='_remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AddField(
            model_name='component',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='engine',
            name='_remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AddField(
            model_name='engine',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='scale',
            name='_remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AddField(
            model_name='scale',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='component',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='engine',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='scale',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='component',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='engine',
            unique_together=set(),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-04 01:51

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0008_auto_20190404_0124'),
    ]

    operations = [
        migrations.AddField(
            model_name='image',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='instanceblueprint',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='instancetemplate',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='location',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-15 05:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0027_remove_group_blueprint'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='hosts',
            field=models.TextField(blank=True, default='', max_length=51200),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-11 23:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0020_clusteroperation_serial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='clusteroperation',
            name='auto',
        ),
        migrations.AddField(
            model_name='clusteroperation',
            name='manual',
            field=models.BooleanField(default=True),
        ),
    ]



# Generated by Django 2.2 on 2019-05-28 02:58

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0005_auto_20190527_2135'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='dataengine',
            name='component',
        ),
        migrations.RemoveField(
            model_name='dataengine',
            name='endpoint',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 07:48

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0024_auto_20190413_0608'),
    ]

    operations = [
        migrations.AddField(
            model_name='step',
            name='uuid',
            field=models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False),
        ),
    ]



# Generated by Django 2.2 on 2019-04-28 04:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0047_auto_20190428_0356'),
    ]

    operations = [
        migrations.AddField(
            model_name='engine',
            name='description',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-12 06:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0018_auto_20190411_2340'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instanceoperation',
            name='manual',
            field=models.BooleanField(default=True, editable=False),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-11 05:28

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0015_instanceoperation_tidied'),
        ('engines', '0017_auto_20190411_0312'),
    ]

    operations = [
        migrations.AddField(
            model_name='clusteroperation',
            name='instances',
            field=models.ManyToManyField(editable=False, to='clouds.Instance'),
        ),
    ]



import importlib
from django.test import TestCase
from clouds.models import Instance

class InstanceMethodTests(TestCase):
    def test_was_mismatch_instance_status_between_db_and_platform(self):
        pass


# Generated by Django 2.2 on 2019-04-28 03:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0046_auto_20190428_0338'),
    ]

    operations = [
        migrations.AddField(
            model_name='component',
            name='stack',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.PROTECT, to='engines.Stack'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='engine',
            name='stack',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.PROTECT, to='engines.Stack'),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 09:38

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0002_auto_20190405_0253'),
        ('clouds', '0012_auto_20190406_0938'),
    ]

    operations = [
	migrations.AlterField(
            model_name='balance',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='clouds.Cloud'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-17 03:29

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0030_auto_20190417_0157'),
        ('engines', '0036_auto_20190417_0157'),
    ]

    operations = [
        migrations.CreateModel(
            name='ClusterGroupOperation',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
            },
            bases=('clouds.groupoperation',),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-15 04:07

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0026_auto_20190415_0228'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='group',
            name='blueprint',
        ),
    ]



# Generated by Django 2.2 on 2019-04-18 07:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0031_group_hosts'),
    ]

    operations = [
        migrations.AlterField(
            model_name='group',
            name='hosts',
            field=models.TextField(blank=True, default='', editable=False, max_length=5120),
        ),
        migrations.AlterField(
            model_name='groupoperation',
            name='script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AlterField(
            model_name='instanceoperation',
            name='script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-03 03:52

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0005_auto_20190402_2147'),
        ('engines', '0002_auto_20190402_2147'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='scaleblueprint',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='scaleblueprint',
            name='instance_blueprints',
        ),
        migrations.RemoveField(
            model_name='scaleblueprint',
            name='owner',
        ),
        migrations.RemoveField(
            model_name='scale',
            name='init_blueprint',
        ),
        migrations.RemoveField(
            model_name='scale',
            name='step_blueprint',
        ),
        migrations.AddField(
            model_name='scale',
            name='init_blueprints',
            field=models.ManyToManyField(related_name='initialized_scales', to='clouds.InstanceBlueprint'),
        ),
        migrations.AddField(
            model_name='scale',
            name='step_blueprints',
            field=models.ManyToManyField(related_name='steped_scales', to='clouds.InstanceBlueprint'),
        ),
        migrations.DeleteModel(
            name='ScaleBlueprint',
        ),
    ]



# Generated by Django 2.2 on 2019-04-28 03:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0045_stack'),
    ]

    operations = [
        migrations.AlterField(
            model_name='component',
            name='endpoint',
            field=models.CharField(blank=True, default='', max_length=200),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-02 21:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0004_auto_20190402_0217'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='instancetemplate',
            options={'verbose_name': 'template'},
        ),
        migrations.RemoveField(
            model_name='mount',
            name='built_time',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-14 07:31

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0022_auto_20190413_1111'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='instance',
            name='remedy_script_todo',
        ),
    ]



# Generated by Django 2.2 on 2019-04-27 08:41

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0037_auto_20190426_1924'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='image',
            unique_together={('cloud', 'name')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-04 01:18

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('clouds', '0006_auto_20190403_2003'),
    ]

    operations = [
        migrations.RenameField(
            model_name='instanceblueprint',
            old_name='remedy_script',
            new_name='_remedy_script',
        ),
        migrations.RenameField(
            model_name='instancetemplate',
            old_name='remedy_script',
            new_name='_remedy_script',
        ),
        migrations.RemoveField(
            model_name='image',
            name='uuid',
        ),
        migrations.RemoveField(
            model_name='location',
            name='uuid',
        ),
        migrations.AddField(
            model_name='instanceblueprint',
            name='name',
            field=models.CharField(default='0', max_length=50),
        ),
        migrations.AddField(
            model_name='instancetemplate',
            name='owner',
            field=models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='instancetemplate',
            name='public',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='location',
            name='_remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AlterField(
            model_name='image',
            name='access_id',
            field=models.CharField(max_length=50, verbose_name='actual id on the location'),
        ),
        migrations.AlterField(
            model_name='image',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='instance',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='instance',
            name='remark',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='instance',
            name='remedy_script_todo',
            field=models.TextField(blank=True, default='', max_length=51200),
        ),
        migrations.AlterField(
            model_name='instance',
            name='uuid',
            field=models.UUIDField(editable=False, null=True, unique=True),
        ),
        migrations.AlterField(
            model_name='instanceblueprint',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='instancetemplate',
            name='access_id',
            field=models.CharField(max_length=50, verbose_name='actual id on the location'),
        ),
        migrations.AlterField(
            model_name='location',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='volume',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='volume',
            name='remark',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
        migrations.AlterField(
            model_name='volume',
            name='uuid',
            field=models.UUIDField(editable=False, null=True, unique=True),
        ),
        migrations.AlterUniqueTogether(
            name='instancetemplate',
            unique_together=set(),
        ),
        migrations.RemoveField(
            model_name='instancetemplate',
            name='uuid',
        ),
    ]



import time
from uuid import uuid4
from enum import Enum
from threading import Thread
from django.db import models
from django.db.models import Q
from django.db import transaction
from django.dispatch import Signal
from django.contrib.auth.models import User
from django.core.validators import MinValueValidator
from django.utils.functional import cached_property
from .base.models import StaticModel, OperatableMixin, OperationModel, M2MOperatableMixin, M2MOperationModel
from django.utils.timezone import now

import pkgutil
from django.conf import settings
drivers=[]
for importer, modname, ispkg in pkgutil.iter_modules((settings.BASE_DIR+'/clouds/drivers',)):
    driver='clouds.drivers.{}'.format(modname)
    drivers.append((driver,driver))

import importlib, json
class Cloud(StaticModel):
    _driver=models.CharField(max_length=50,choices=drivers)
    _platform_credential=models.TextField(max_length=5120,blank=True,null=True)
    _instance_credential=models.TextField(max_length=2048,blank=True,null=True)    
    hosts=models.TextField(max_length=5120,blank=True,null=True)
    owner=models.ForeignKey(User,on_delete=models.PROTECT,editable=False,verbose_name='admin')
    @cached_property
    def driver(self):
        return importlib.import_module(self._driver)
    @cached_property
    def platform_credential(self):
        return json.loads(self._platform_credential)
    @cached_property
    def instance_credential(self):
        return json.loads(self._instance_credential)
    def import_image(self):
        for img in self.driver.image_list(self.platform_credential):
            name=img['name']
            id=img['id']
            if Image.objects.filter(cloud=self, access_id=id).exists(): continue
            if Image.objects.filter(cloud=self, name=name).exists():
                name='{}-{}'.format(name,img['id'])
            Image(
                name=name,
                cloud=self,
                access_id = img['id'],
                hostname='packone',
                owner=self.owner,
                remark='auto imported',
                enabled=self.enabled,
                public=self.public
            ).save()
    def import_template(self):
        for tpl in self.driver.template_list(self.platform_credential):
            id=tpl['id']
            if InstanceTemplate.objects.filter(cloud=self, access_id=id).exists(): continue
            InstanceTemplate(
                access_id=id,
                name=tpl['name'],
                mem=tpl['mem'],
                vcpu=tpl['vcpu'],
                cloud=self,
                owner=self.owner,
                remark='auto imported',
                enabled=self.enabled,
                public=self.public
            ).save()
            
def clouds_of_user(self):
    return Cloud.objects.filter(
        balance__in=self.balances(),
        enabled=True,
    ).filter(Q(public=True) | Q(owner=self)).distinct()
User.clouds=clouds_of_user

class Image(StaticModel):
    name=models.CharField(max_length=50)
    cloud=models.ForeignKey(Cloud,on_delete=models.PROTECT)
    access_id = models.CharField(max_length=50,verbose_name="actual id on the cloud")
    hostname=models.CharField(max_length=50,default='packone')
    parent=models.ForeignKey("self",on_delete=models.PROTECT,blank=True,null=True)
    class Meta:
        unique_together = ('cloud', 'name')
    @cached_property
    def remedy_script(self):
        s="###remedy image: {}###\n{}\n".format(self.name,self._remedy_script) if self._remedy_script else ""
        if self.parent:
            s=self.parent.remedy_script+s
        return s
    def launch(self, template, owner, remedy_script='', number=1, remark=None):
        hostname=self.hostname
        if number>1:
            parts=hostname.split('.')
            hostname='.'.join((parts[0]+str(number),'.'.join(parts[1:])))
        ins=Instance(
            cloud=self.cloud,
            template=template,
            image=self,
            hostname=hostname,
            remedy_script_todo=remedy_script,
            owner=owner,
            remark=remark
        )
        ins.save()
        return ins

class InstanceTemplate(StaticModel):#TODO support root volume resize
    name=models.CharField(max_length=50)
    cloud=models.ForeignKey(Cloud,on_delete=models.PROTECT)
    access_id = models.CharField(max_length=50,verbose_name="actual id on the cloud")
    vcpu=models.PositiveIntegerField(default=1, validators=[MinValueValidator(1)])
    mem=models.PositiveIntegerField(default=512,validators=[MinValueValidator(256)])
    class Meta:
        verbose_name = "template"
        unique_together = ('cloud', 'name')
    def __str__(self):
        return "{}/vcpu:{},mem:{}".format(self.name,self.vcpu,self.mem)

class InstanceBlueprint(StaticModel):
    name=models.CharField(max_length=50)
    cloud=models.ForeignKey(Cloud,on_delete=models.PROTECT)
    template=models.ForeignKey(InstanceTemplate,on_delete=models.PROTECT)
    image=models.ForeignKey(Image,on_delete=models.PROTECT,related_name="instance_blueprints")
    volume_capacity=models.IntegerField(validators=[MinValueValidator(0)],default=0)
    volume_mount_point=models.CharField(default="/data",max_length=100)
    quantity=models.PositiveIntegerField(default=1, validators=[MinValueValidator(1)])
    class Meta:
        verbose_name = "blueprint"
        unique_together = ('cloud', 'name')
    def __str__(self):
        return "{}/{}/{}".format(self.image,self.template,self.quantity)
    @transaction.atomic
    def launch(self, owner, next_number=1, remark=None):
        if not remark: remark='launched from instance_blueprint: {}'.format(self)
        inss=[]
        for number in range(next_number,next_number+self.quantity):
            ins=self.image.launch(
                template=self.template,
                owner=owner,
                remedy_script=self.remedy_script,
                number=number,
                remark=remark
            )
            inss.append(ins)
            if self.volume_capacity:
                volume=Volume(
                    cloud=self.cloud,
                    capacity=self.volume_capacity,
                    owner=owner,
                    remark=remark
                )
                volume.save()
                Mount(
                    point=self.volume_mount_point,
                    instance=ins,
                    volume=volume
                ).save()
        return inss

def blueprints_of_user(self):
    return InstanceBlueprint.objects.filter(
        cloud__in=self.clouds(),
        enabled=True
    ).filter(Q(public=True) | Q(owner=self))
User.blueprints=blueprints_of_user

#TODO class VLan

class INSTANCE_STATUS(Enum):#greater value means worse status
    null=0 #unknown
    active=1
    block=2
    suspend=3
    shutdown=4
    poweroff=5
    breakdown=6
    pause=7
    failure=8 #libvirt reserv code
    host_lost=9
    instance_lost=10
    building=11
        
monitored = Signal(providing_args=["instance","name"])

class Instance(models.Model,OperatableMixin):
    ipv4=models.GenericIPAddressField(protocol='IPv4',blank=True,null=True,editable=False)
    ipv6=models.GenericIPAddressField(protocol='IPv6',blank=True,null=True,editable=False)
    cloud=models.ForeignKey(Cloud,on_delete=models.PROTECT)
    uuid=models.UUIDField(unique=True,auto_created=False,null=True,editable=False)
    image=models.ForeignKey(Image,on_delete=models.PROTECT)
    template=models.ForeignKey(InstanceTemplate,on_delete=models.PROTECT)
    hostname=models.CharField(max_length=50,blank=True,null=True)#TODO not null, unique and templated
    remedy_script_todo=models.TextField(max_length=51200,default="",blank=True)
    created_time=models.DateTimeField(auto_now_add=True)
    built_time=models.DateTimeField(blank=True,null=True,editable=False)
    remark = models.CharField(blank=True,null=True,max_length=100)
    owner=models.ForeignKey(User,on_delete=models.PROTECT,editable=False)
    status= models.PositiveIntegerField(choices=[(status.value,status.name) for status in INSTANCE_STATUS],default=INSTANCE_STATUS.building.value,editable=False)
    deleting = models.BooleanField(default=False,editable=False)
    def __str__(self):
        return "{}/{}/{}".format(self.image.name,self.template.name,self.ipv4)
    @staticmethod
    def get_operation_model():
        return InstanceOperation
    @property
    def ready(self):
        return self.uuid
    @property
    def building(self):
        return self.built_time and not self.ready
    @cached_property
    def vnc_url(self):
        return self.cloud.driver.vm_vnc_url(self.cloud.platform_credential,str(self.uuid))
    @cached_property
    def hosts_record(self):
        if not self.ready: raise Exception('instance not ready')
        hostname_parts=self.hostname.split('.')
        hostnames=' '.join(('.'.join(hostname_parts[0:pi+1]) for pi in range(0,len(hostname_parts))))		
        return self.ipv4+' '+hostnames
    @property
    def mountable(self):
        #TODO get mount condition from driver
        if self.status in [
            INSTANCE_STATUS.poweroff.value,
            INSTANCE_STATUS.shutdown.value
        ]: return True
        if self.status==INSTANCE_STATUS.building.value and self.ready: return True
        return False
    @property
    def umountable(self):
        return self.mountable
    def monitor(self,notify=True):
        if not self.ready: raise Exception('instance not ready')
        status = self.cloud.driver.vm_status(self.cloud.platform_credential,str(self.uuid))
        if self.__class__.objects.filter(pk=self.pk).update(status=status):
            self.refresh_from_db()
            if notify: monitored.send(sender=self.__class__, instance=self, name='monitored')
    
class VOLUME_STATUS(Enum):#greater value means worse status
    null=0 #unknown
    available=1
    mounted=2
    building=3 #executing remedy_script

class Volume(models.Model):
    cloud=models.ForeignKey(Cloud,on_delete=models.PROTECT)
    uuid=models.UUIDField(unique=True,auto_created=False,null=True,editable=False)
    capacity=models.IntegerField()
    created_time=models.DateTimeField(auto_now_add=True)
    built_time=models.DateTimeField(blank=True,null=True,editable=False)
    remark = models.CharField(blank=True,null=True,max_length=100)
    owner=models.ForeignKey(User,on_delete=models.PROTECT,editable=False)
    status= models.PositiveIntegerField(choices=[(status.value,status.name) for status in VOLUME_STATUS],default=VOLUME_STATUS.building.value,editable=False)
    deleting = models.BooleanField(default=False,editable=False)
    def __str__(self):
        return "{}/{}/{}".format(self.cloud.name,self.capacity,str(self.uuid).split('-')[0])
    @property
    def ready(self):
        return self.uuid
    @property
    def building(self):
        return self.built_time and not self.ready
    @property
    def mountable(self):
        #TODO get mount condition from driver
        return self.status == VOLUME_STATUS.available.value
    @property
    def umountable(self):
        return self.status == VOLUME_STATUS.mounted.value
    def get_running_operations(self):
        return Mount.objects.filter(
            volume=self,
            dev__isnull=True,
            completed_time__isnull=False
        ).order_by('id')

#TODO add same-cloud check
class Mount(models.Model):
    volume=models.OneToOneField(Volume,on_delete=models.PROTECT)
    instance=models.ForeignKey(Instance,on_delete=models.PROTECT)
    dev=models.CharField(max_length=100,null=True,editable=False)
    point=models.CharField(default="/data",max_length=200)
    created_time=models.DateTimeField(auto_now_add=True)
    completed_time=models.DateTimeField(blank=True,null=True,editable=False)
    class Meta:
        unique_together = ('instance', 'point')
        default_permissions=('add', 'delete', 'view')
    def __str__(self):
        return "{}/{}/{}".format(self.instance,self.dev,self.point)
    @property
    def ready(self):
        return self.dev
    @property
    def executing(self):
        return self.completed_time and not self.ready

class INSTANCE_OPERATION(Enum):
    start="start"
    reboot="reboot"
    shutdown="shutdown"
    poweroff="poweroff"
    remedy="remedy"

class OPERATION_STATUS(Enum):
    success="success"
    failed="failed"
    running="running"
    waiting="waiting"
    
executed = Signal(providing_args=["instance","name"])

class InstanceOperation(OperationModel):
    target=models.ForeignKey(Instance,on_delete=models.CASCADE)
    log=models.TextField(max_length=51200,null=True,editable=False)
    def execute(self):
        def perform(self=self):
            from . import utils
            if self.operation!=INSTANCE_OPERATION.remedy.value:
                try:
                    output=self.target.cloud.driver.vm_op(
                        self.target.cloud.platform_credential,
                        str(self.target.uuid),
                        self.operation
                    )
                    if self.is_boot:
                        utils.SSH(self.target.ipv4,
                            self.target.cloud.instance_credential
                        ).close() #to wait booting finished
                except Exception as e:
                    self.status=OPERATION_STATUS.failed.value
                    self.log='EXCEPTION MESSAGE:\n'+str(e)
                else:
                    self.status=OPERATION_STATUS.success.value
                    self.log=output
            else:
                if not self.script:
                    self.log='EXCEPTION MESSAGE:\nblank remedy script'
                    self.status=OPERATION_STATUS.failed.value
                else:
                    try:
                        ssh=utils.SSH(self.target.ipv4, self.target.cloud.instance_credential)
                        out, err = ssh.exec_batch(self.script)
                        self.log=out+err
                        self.status=OPERATION_STATUS.failed.value if err else OPERATION_STATUS.success.value
                    except Exception as e:
                        self.log='EXCEPTION MESSAGE:\n'+str(e)
                        self.status=OPERATION_STATUS.failed.value
            self.completed_time=now()
            self.save()
            executed.send(sender=InstanceOperation, instance=self, name='executed')
        Thread(target=perform).start()

class Group(models.Model,M2MOperatableMixin):
    uuid=models.UUIDField(auto_created=True, default=uuid4, editable=False)
    instances=models.ManyToManyField(Instance,blank=True,editable=False)
    remedy_script_todo=models.TextField(max_length=51200,default="",blank=True)
    destroy_script_todo=models.TextField(max_length=51200,default="",blank=True)
    hosts=models.TextField(max_length=5120,default="",blank=True,editable=False)
    remark = models.CharField(blank=True,null=True,max_length=100)
    owner=models.ForeignKey(User,on_delete=models.PROTECT,editable=False)
    created_time=models.DateTimeField(auto_now_add=True)
    built_time=models.DateTimeField(blank=True, null=True, editable=False)
    status= models.PositiveIntegerField(choices=[(status.value,status.name) for status in INSTANCE_STATUS],default=INSTANCE_STATUS.building.value,editable=False)
    deleting = models.BooleanField(default=False,editable=False)
    class Meta:
        ordering = ['pk']
    def __str__(self):
        return self.long_id
    @property
    def long_id(self):
        return str(self.uuid).split('-')[0]
    @staticmethod
    def get_operation_model():
        return GroupOperation
    @property
    def operatables(self):
        return self.instances.all()
    @property
    def ready(self):
        return self.built_time
    @property
    def building(self):
        return self.built_time and not self.ready
    @property
    def mounts(self):
        return Mount.objects.filter(instance__in=self.instances.all())
    @transaction.atomic
    def delete(self, *args, **kwargs):
        if not self.ready:
            print('WARNNING: delete {} under building'.format(self._meta.verbose_name))
        elif self.destroy_script_todo:
            self.remedy(self.destroy_script_todo)
        operatables=self.operatables
        if operatables.exists():
            self.deleting=True
            self.save()
            for operatable in operatables:
                operatable.deleting=True
                operatable.save()
                for m in operatable.mount_set.select_related('volume').select_for_update():
                    if m.ready:
                        m.volume.deleting=True
                        m.volume.save()
                    else:
                        m.delete()
                InstanceOperation(
                    target=operatable,
                    operation=INSTANCE_OPERATION.poweroff.value,
                    manual=True
                ).save()
                time.sleep(0.1)
        else:
            super().delete(*args, **kwargs)

class GroupOperation(M2MOperationModel):
    target=models.ForeignKey(Group,on_delete=models.CASCADE)
    def __str__(self):
        return "{}({}/{})".format(self.batch,self.operation,self.status)
    class Meta(M2MOperationModel.Meta):
        verbose_name = "group operation"
    @staticmethod
    def get_sub_operation_model():
        return InstanceOperation


# Generated by Django 2.2 on 2019-04-26 19:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0036_group_destroy_script_todo'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='image',
            unique_together=set(),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-15 01:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0025_auto_20190415_0119'),
        ('engines', '0034_auto_20190414_2200'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='step',
            name='cluster',
        ),
        migrations.RemoveField(
            model_name='step',
            name='instances',
        ),
        migrations.RemoveField(
            model_name='step',
            name='owner',
        ),
        migrations.RemoveField(
            model_name='stepoperation',
            name='serial',
        ),
        migrations.RemoveField(
            model_name='stepoperation',
            name='target',
        ),
        migrations.AddField(
            model_name='cluster',
            name='steps',
            field=models.ManyToManyField(blank=True, editable=False, to='clouds.Group'),
        ),
        migrations.AlterField(
            model_name='cluster',
            name='built_time',
            field=models.DateTimeField(blank=True, editable=False, null=True),
        ),
        migrations.DeleteModel(
            name='Step',
        ),
        migrations.DeleteModel(
            name='StepOperation',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-14 07:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0032_auto_20190414_0731'),
    ]

    operations = [
        migrations.AddField(
            model_name='cluster',
            name='remedy_script_todo',
            field=models.TextField(blank=True, default='', max_length=51200),
        ),
        migrations.AddField(
            model_name='step',
            name='remedy_script_todo',
            field=models.TextField(blank=True, default='', max_length=51200),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 07:56

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0026_auto_20190413_0750'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='clusteroperation',
            name='steps',
        ),
        migrations.AddField(
            model_name='clusteroperation',
            name='step',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='engines.Step'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 07:50

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0025_step_uuid'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='step',
            name='uuid',
        ),
        migrations.AddField(
            model_name='step',
            name='built_time',
            field=models.DateTimeField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='step',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-12 09:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0022_auto_20190412_0637'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='clusteroperation',
            name='broadcast',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-04 23:47

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0010_auto_20190404_2346'),
    ]

    operations = [
        migrations.AlterField(
            model_name='volume',
            name='status',
            field=models.PositiveIntegerField(choices=[(0, 'null'), (1, 'available'), (2, 'mounted'), (3, 'building')], default=3, editable=False),
        ),
    ]



# Generated by Django 2.2 on 2019-04-28 22:31

from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('engines', '0048_engine_description'),
    ]

    operations = [
        migrations.AlterField(
            model_name='component',
            name='name',
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name='engine',
            name='name',
            field=models.CharField(max_length=50),
        ),
        migrations.AlterField(
            model_name='stack',
            name='_driver',
            field=models.CharField(choices=[('engines.drivers.ambari', 'engines.drivers.ambari')], max_length=50),
        ),
        migrations.AlterUniqueTogether(
            name='component',
            unique_together={('stack', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='engine',
            unique_together={('stack', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='stack',
            unique_together={('owner', 'name')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-17 01:57

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0029_remove_group_hosts'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='groupoperation',
            options={'verbose_name': 'group operation'},
        ),
    ]



# Generated by Django 2.2 on 2019-04-17 05:26

from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0030_auto_20190417_0157'),
        ('engines', '0037_clustergroupoperation'),
    ]

    operations = [
        migrations.CreateModel(
            name='StepOperation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('batch_uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('operation', models.CharField(choices=[('start', 'start'), ('reboot', 'reboot'), ('shutdown', 'shutdown'), ('poweroff', 'poweroff'), ('remedy', 'remedy')], default='start', max_length=50)),
                ('script', models.TextField(blank=True, max_length=5120, null=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('started_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('completed_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.CharField(choices=[('success', 'success'), ('failed', 'failed'), ('running', 'running'), ('waiting', 'waiting')], default='running', editable=False, max_length=50)),
                ('tidied', models.BooleanField(default=False, editable=False)),
                ('manual', models.BooleanField(default=True, editable=False)),
                ('serial', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='engines.StepOperation')),
            ],
            options={
                'verbose_name': 'operation',
                'abstract': False,
            },
        ),
        migrations.DeleteModel(
            name='ClusterGroupOperation',
        ),
        migrations.CreateModel(
            name='Step',
            fields=[
            ],
            options={
                'verbose_name': 'Step',
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('clouds.group',),
        ),
        migrations.AlterModelOptions(
            name='clusteroperation',
            options={'verbose_name': 'cluster operation'},
        ),
        migrations.AlterField(
            model_name='cluster',
            name='steps',
            field=models.ManyToManyField(blank=True, editable=False, to='engines.Step'),
        ),
        migrations.AddField(
            model_name='stepoperation',
            name='target',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='engines.Step'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-30 23:04

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0002_auto_20190330_2129'),
    ]

    operations = [
        migrations.AddField(
            model_name='instanceoperation',
            name='started_time',
            field=models.DateTimeField(blank=True, editable=False, null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 08:33

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0027_auto_20190413_0756'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='step',
            options={'ordering': ['pk']},
        ),
        migrations.RemoveField(
            model_name='cluster',
            name='steps',
        ),
        migrations.AddField(
            model_name='step',
            name='cluster',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='steps', to='engines.Cluster'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-17 06:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0030_auto_20190417_0157'),
        ('engines', '0038_auto_20190417_0526'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='stepoperation',
            name='serial',
        ),
        migrations.RemoveField(
            model_name='stepoperation',
            name='target',
        ),
        migrations.DeleteModel(
            name='Step',
        ),
        migrations.AlterField(
            model_name='cluster',
            name='steps',
            field=models.ManyToManyField(blank=True, editable=False, to='clouds.Group'),
        ),
        migrations.DeleteModel(
            name='StepOperation',
        ),
        migrations.CreateModel(
            name='StepOperation',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('clouds.groupoperation',),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-02 02:21

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('clouds', '0004_auto_20190402_0217'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Cluster',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50)),
                ('current_step', models.PositiveIntegerField(default=0, validators=[django.core.validators.MinValueValidator(0)])),
                ('remedy_script_todo', models.TextField(blank=True, default='', max_length=5120)),
                ('public', models.BooleanField(default=False)),
                ('remark', models.CharField(blank=True, max_length=2000, null=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('built_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.PositiveIntegerField(choices=[(0, 'null'), (1, 'active'), (2, 'block'), (3, 'suspend'), (4, 'shutdown'), (5, 'poweroff'), (6, 'breakdown'), (7, 'pause'), (8, 'failure'), (9, 'host_lost'), (10, 'instance_lost'), (11, 'building')], default=11, editable=False)),
            ],
        ),
        migrations.CreateModel(
            name='ClusterOperation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('operation', models.CharField(choices=[('start', 'start'), ('reboot', 'reboot'), ('shutdown', 'shutdown'), ('poweroff', 'poweroff'), ('remedy', 'remedy')], default='start', max_length=50)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('status', models.CharField(choices=[('success', 'success'), ('failed', 'failed'), ('running', 'running'), ('waiting', 'waiting')], default='running', editable=False, max_length=50)),
                ('started_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('completed_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('target', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='engines.Cluster')),
            ],
            options={
                'verbose_name': 'operations',
            },
        ),
        migrations.CreateModel(
            name='Component',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('type', models.CharField(choices=[('master', 'master'), ('slave', 'slave'), ('client', 'client')], max_length=50)),
                ('endpoint', models.CharField(max_length=200)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('images', models.ManyToManyField(to='clouds.Image')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Engine',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('components', models.ManyToManyField(to='engines.Component')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Scale',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('auto', models.BooleanField(default=False)),
                ('hosts', models.TextField(blank=True, max_length=5120, null=True)),
                ('remedy_script', models.TextField(blank=True, max_length=5120, null=True)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='ScaleBlueprint',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('remedy_script', models.TextField(blank=True, max_length=5120, null=True)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('instance_blueprints', models.ManyToManyField(to='clouds.InstanceBlueprint')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'blueprint',
            },
        ),
        migrations.AddField(
            model_name='scale',
            name='init_blueprint',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='initialized_scales', to='engines.ScaleBlueprint'),
        ),
        migrations.AddField(
            model_name='scale',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='scale',
            name='step_blueprint',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, related_name='steped_scales', to='engines.ScaleBlueprint'),
        ),
        migrations.AddField(
            model_name='cluster',
            name='engines',
            field=models.ManyToManyField(to='engines.Engine'),
        ),
        migrations.AddField(
            model_name='cluster',
            name='instances',
            field=models.ManyToManyField(to='clouds.Instance'),
        ),
        migrations.AddField(
            model_name='cluster',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='cluster',
            name='scale',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='engines.Scale'),
        ),
        migrations.AlterUniqueTogether(
            name='scaleblueprint',
            unique_together={('name', 'owner')},
        ),
        migrations.AlterUniqueTogether(
            name='engine',
            unique_together={('name', 'owner')},
        ),
        migrations.AlterUniqueTogether(
            name='component',
            unique_together={('name', 'owner')},
        ),
        migrations.AlterUniqueTogether(
            name='cluster',
            unique_together={('name', 'owner')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 09:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0013_auto_20190406_0952'),
        ('user', '0003_auto_20190406_0938'),
    ]

    operations = [
        migrations.RenameField(
            model_name='balance',
            old_name='location',
            new_name='cloud',
        ),
        migrations.AlterUniqueTogether(
            name='balance',
            unique_together={('profile', 'cloud')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 08:36

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0028_auto_20190413_0833'),
    ]

    operations = [
        migrations.AlterField(
            model_name='step',
            name='cluster',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='steps', to='engines.Cluster'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-26 18:39

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0035_auto_20190420_0247'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='destroy_script_todo',
            field=models.TextField(blank=True, default='', max_length=51200),
        ),
    ]



# Generated by Django 2.2 on 2019-06-25 20:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0013_auto_20190617_0430'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datainstance',
            name='status',
            field=models.PositiveIntegerField(choices=[(0, 'loading'), (1, 'running'), (2, 'stop')], default=0, editable=False),
        ),
    ]



# Generated by Django 2.2 on 2019-06-14 20:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0008_dataset_uri'),
    ]

    operations = [
        migrations.AddField(
            model_name='dataengine',
            name='uri_prefix',
            field=models.CharField(default='test', max_length=2000),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-30 20:28

from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('access_id', models.CharField(max_length=50)),
                ('name', models.CharField(max_length=50)),
                ('hostname', models.CharField(default='packone', max_length=50)),
                ('_remedy_script', models.TextField(default='', max_length=5120)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='Instance',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(editable=False, null=True)),
                ('hostname', models.CharField(blank=True, max_length=50, null=True)),
                ('ipv4', models.GenericIPAddressField(blank=True, editable=False, null=True, protocol='IPv4')),
                ('ipv6', models.GenericIPAddressField(blank=True, editable=False, null=True, protocol='IPv6')),
                ('remedy_script_todo', models.TextField(default='', max_length=5120)),
                ('monitoring', models.BooleanField(default=False, editable=False)),
                ('remark', models.CharField(blank=True, max_length=2000, null=True)),
                ('created_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.PositiveIntegerField(choices=[(0, 'null'), (1, 'active'), (2, 'block'), (3, 'suspend'), (4, 'shutdown'), (5, 'poweroff'), (6, 'breakdown'), (7, 'pause'), (8, 'failure'), (9, 'host_lost'), (10, 'instance_lost'), (11, 'building')], default=11, editable=False)),
                ('deleting', models.BooleanField(default=False, editable=False)),
                ('image', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Image')),
            ],
        ),
        migrations.CreateModel(
            name='InstanceBlueprint',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('volume_capacity', models.IntegerField(default=0, validators=[django.core.validators.MinValueValidator(0)])),
                ('volume_mount_point', models.CharField(default='/data', max_length=100)),
                ('quantity', models.PositiveIntegerField(default=1, validators=[django.core.validators.MinValueValidator(1)])),
                ('remedy_script', models.TextField(default='', max_length=5120)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('image', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='instance_blueprints', to='clouds.Image')),
            ],
            options={
                'verbose_name': 'blueprints',
            },
        ),
        migrations.CreateModel(
            name='InstanceOperation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('operation', models.CharField(choices=[('start', 'start'), ('reboot', 'reboot'), ('shutdown', 'shutdown'), ('poweroff', 'poweroff'), ('remedy', 'remedy')], default='start', max_length=50)),
                ('batch_uuid', models.UUIDField(editable=False, null=True)),
                ('script', models.TextField(blank=True, max_length=5120, null=True)),
                ('log', models.TextField(blank=True, max_length=51200, null=True)),
                ('created_time', models.DateTimeField(auto_now_add=True, null=True)),
                ('status', models.CharField(choices=[('success', 'success'), ('failed', 'failed'), ('running', 'running'), ('waiting', 'waiting')], default='running', editable=False, max_length=50)),
                ('completed_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('target', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='clouds.Instance')),
            ],
            options={
                'verbose_name': 'operations',
            },
        ),
        migrations.CreateModel(
            name='InstanceTemplate',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('access_id', models.CharField(max_length=50, unique=True)),
                ('name', models.CharField(max_length=50)),
                ('vcpu', models.PositiveIntegerField(default=1, validators=[django.core.validators.MinValueValidator(1)])),
                ('mem', models.PositiveIntegerField(default=512, validators=[django.core.validators.MinValueValidator(256)])),
                ('remedy_script', models.TextField(blank=True, max_length=5120, null=True)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
            ],
        ),
        migrations.CreateModel(
            name='Location',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('platform', models.CharField(max_length=50)),
                ('_platform_credential', models.TextField(blank=True, max_length=5120, null=True)),
                ('_instance_credential', models.TextField(blank=True, max_length=2048, null=True)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Mount',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('dev', models.CharField(editable=False, max_length=100, null=True)),
                ('point', models.CharField(default='/data', max_length=200)),
                ('created_time', models.DateTimeField(auto_now_add=True, null=True)),
                ('completed_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('deleting', models.BooleanField(default=False, editable=False)),
                ('instance', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Instance')),
            ],
            options={
                'default_permissions': ('add', 'delete', 'view'),
            },
        ),
        migrations.CreateModel(
            name='Volume',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(editable=False, null=True)),
                ('capacity', models.IntegerField()),
                ('remark', models.CharField(blank=True, max_length=2000, null=True)),
                ('created_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.PositiveIntegerField(choices=[(0, 'null'), (1, 'available'), (2, 'mounted'), (3, 'building')], default=3, editable=False)),
                ('deleting', models.BooleanField(default=False, editable=False)),
                ('location', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Location')),
                ('owner', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='mount',
            name='volume',
            field=models.OneToOneField(on_delete=django.db.models.deletion.PROTECT, to='clouds.Volume'),
        ),
        migrations.AddField(
            model_name='instancetemplate',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Location'),
        ),
        migrations.AddField(
            model_name='instanceblueprint',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Location'),
        ),
        migrations.AddField(
            model_name='instanceblueprint',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='instanceblueprint',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.InstanceTemplate'),
        ),
        migrations.AddField(
            model_name='instance',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Location'),
        ),
        migrations.AddField(
            model_name='instance',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='instance',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.InstanceTemplate'),
        ),
        migrations.AddField(
            model_name='image',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Location'),
        ),
        migrations.AddField(
            model_name='image',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='image',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='clouds.Image'),
        ),
        migrations.AlterUniqueTogether(
            name='mount',
            unique_together={('instance', 'point')},
        ),
        migrations.AlterUniqueTogether(
            name='instancetemplate',
            unique_together={('location', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='image',
            unique_together={('location', 'name')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-14 22:00

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('engines', '0033_auto_20190414_0736'),
    ]

    operations = [
        migrations.AddField(
            model_name='step',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='step',
            name='owner',
            field=models.ForeignKey(default=1, editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='step',
            name='remark',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]



# Generated by Django 2.2 on 2019-04-20 02:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0032_auto_20190418_0732'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='instanceoperation',
            options={'ordering': ['-started_time', '-pk'], 'verbose_name': 'operation'},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-10 22:20

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0014_auto_20190410_2050'),
    ]

    operations = [
        migrations.RenameField(
            model_name='clusteroperation',
            old_name='uuid',
            new_name='batch_uuid',
        ),
        migrations.AddField(
            model_name='clusteroperation',
            name='tidied',
            field=models.BooleanField(default=False, editable=False),
        ),
    ]



# Generated by Django 2.2 on 2019-04-07 23:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0010_auto_20190406_2245'),
    ]

    operations = [
        migrations.AddField(
            model_name='cluster',
            name='deleting',
            field=models.BooleanField(default=False, editable=False),
        ),
    ]



from secrets import token_urlsafe
from django.dispatch import receiver
from django.db.models.signals import pre_save, post_save, post_delete, pre_delete
from .import models

@receiver(pre_save, sender=models.Credential)
def credential_pre_save(sender, instance, **kwargs):
    if not instance.pk:
        if not instance.ssh_passwd:
            instance.ssh_passwd = token_urlsafe(15)

@receiver(pre_delete, sender=models.Credential)
def credential_pre_delete(sender, instance, **kwargs):
    if instance.ssh_user=='root':
        raise Exception('cannot delete credentials of root')

@receiver(pre_save, sender=models.Profile)
def profile_pre_save(sender, instance, **kwargs):
    if instance.enabled:
        sender.objects.filter(enabled=True,owner=instance.owner).update(enabled=False)



# Generated by Django 2.2 on 2019-06-14 20:13

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0009_dataengine_uri_prefix'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dataengine',
            name='uri_prefix',
            field=models.CharField(blank=True, default='', max_length=2000),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-11 11:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0016_instanceoperation_auto'),
    ]

    operations = [
        migrations.AddField(
            model_name='instanceoperation',
            name='serial',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='clouds.InstanceOperation'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-15 02:28

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0025_auto_20190415_0119'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='blueprint',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.PROTECT, to='clouds.InstanceBlueprint'),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='instanceoperation',
            name='target',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='clouds.Instance'),
            preserve_default=False,
        ),
    ]



import coreapi
from django.conf import settings
#TODO directly use libvirt and ceph
def do_action(credential, action, params):
    client=coreapi.Client(auth=coreapi.auth.BasicAuthentication(credential['user'], credential['passwd']))
    schema = client.get(credential['api_endpoint'])#TODO improve perf.
    return client.action(schema, action, params=params)

def vm_op(credential, vm_id, op):
    action = ["vms","operations","partial_update"]
    params = {
        "vm_id":vm_id,
        "op":op
    }
    try:
        return do_action(credential, action,params)
    except coreapi.exceptions.ErrorMessage as e:
        if 'not running' in e.error._data['detail'] or 'already running' in e.error._data['detail']:
            print(e.error._data['detail'])
            return True
        else:
            raise e

def vm_status(credential, vm_id):
    action = ["vms","status","list"]
    params = {
        "vm_id":vm_id
    }
    try:
        return int(do_action(credential, action,params))
    except coreapi.exceptions.ErrorMessage as e:
        if '\u865a\u62df\u673aUUID\u9519\u8bef' in e.error._data['detail']:
            print('VM UUID Mismatch')
            return 5
        else:
            raise e

def vm_vnc_url(credential, vm_id):
    action = ["vms","vnc","create"]
    params = {
        "vm_id":vm_id
    }
    return do_action(credential,action,params)['url']

def vm_list(credential, include_remarks=[]):
    include_remarks.append(settings.PACKONE_LABEL)
    action = ["vms", "list"]
    params = {
        "group_id":credential['group_id'],
    }
    result=do_action(credential, action, params)
    vms = list(filter(lambda x: len(set(x['remarks'].split(';')).intersection(set(include_remarks)))==len(include_remarks), result))
    return vms

def vm_read(credential, vm_id):
    action = ["vms","read"]
    params = {
        "vm_id":vm_id
    }
    vm_info=do_action(credential, action, params)
    if not vm_info['remarks'].startswith(settings.PACKONE_LABEL):
        raise Exception('Cannot retrieve non-{} vms!'.format(settings.PACKONE_LABEL))
    else:
        #strip beginning settings.PACKONE_LABEL
        vm_info['remark']=vm_info['remarks'][len(settings.PACKONE_LABEL):]
        return vm_info

def vm_create(credential,image_id,vcpu,mem,template_id=None,remark=''):
    action = ["vms","create"]
    params = {
        "image_id":int(image_id),
        "vcpu":vcpu,
        "mem":mem,
        "group_id":credential['group_id'],
        'vlan_id':credential['vlan_id'],
        "remarks":remark,
    }
    vm_id=do_action(credential, action, params)
    return vm_read(credential, vm_id)

def vm_delete(credential, vm_id):
    action = ["vms","delete"]
    params = {
        "vm_id":vm_id,
    }
    try:
        return do_action(credential, action,params)
    except coreapi.exceptions.ErrorMessage as e:
        if '\u865a\u62df\u673aUUID\u9519\u8bef' in e.error._data['detail']:
            print('VM UUID Mismatch')
            return True
        else:
            raise e

def vm_force_delete(credential, vm_id):
    try:
        vm_op(credential, vm_id, 'poweroff')
    except Exception as e:
        print(e)
    return vm_delete(credential, vm_id)

def volume_op(credential, volume_id, vm_id, op):
    action = ["volumes","vm","partial_update"]
    params = {
        "volume_id":volume_id,
        "vm_id":vm_id,
        "op":op
    }
    return do_action(credential, action, params)

def volume_mount(credential, volume_id, vm_id):
    volume_op(credential, volume_id, vm_id, 'mount')
    return volume_read(credential, volume_id)

def volume_unmount(credential, volume_id, vm_id):
    try:
        return volume_op(credential, volume_id, vm_id, 'unmount')
    except coreapi.exceptions.ErrorMessage as e:
        if '\u865a\u62df\u673aUUID\u9519\u8bef' in e.error._data['detail']:
            print('VM UUID Mismatch')
            return True
        else:
            raise e

def volume_read(credential, volume_id):
    action = ["volumes","read"]
    params = {
        "volume_id":volume_id
    }
    volume_info=do_action(credential, action,params)
    if not volume_info['remarks'].startswith(settings.PACKONE_LABEL):
        raise Exception('Cannot retrieve non-{} volumes!'.format(settings.PACKONE_LABEL))
    else:
        #strip beginning settings.PACKONE_LABEL
        volume_info['remark']=volume_info['remarks'][len(settings.PACKONE_LABEL):]
        volume_info['uuid']=volume_info['id']
        if volume_info['dev']:
            volume_info['dev']='/dev/'+volume_info['dev']
        return volume_info

def volume_create(credential, size, remark=''):
    action = ["volumes","create"]
    params = {
        "group_id":credential['group_id'],
        "cephpool_id":credential['volume_poll_id'],
        'size':size,
        "remarks":remark
    }
    volume_id=do_action(credential, action, params)
    return volume_read(credential, volume_id)

def volume_delete(credential, volume_id):
    action = ["volumes","delete"]
    params = {
        "volume_id":volume_id,
    }
    try:
        return do_action(credential, action, params)
    except coreapi.exceptions.ErrorMessage as e:
        if 'CEPH\u5757ID\u6709\u8bef' in e.error._data['detail']:
            print('CEPH UUID Mismatch')
            return True
        else:
            raise e

def image_list(credential):
    action = ["images","list"]
    params = {
        "pool_id":credential['image_poll_id'],
    }
    return do_action(credential, action, params)

def template_list(credential):
    return (
        {'id': '0', 'name': 'm1.nano', 'mem': 64, 'vcpu': 1}, 
        {'id': '1', 'name': 'm1.tiny', 'mem': 512, 'vcpu': 1}, 
        {'id': '2', 'name': 'm1.small', 'mem': 2048, 'vcpu': 1},
        {'id': '3', 'name': 'm1.medium', 'mem': 4096, 'vcpu': 2},
        {'id': '4', 'name': 'm1.large', 'mem': 8192, 'vcpu': 2}, 
        {'id': '5', 'name': 'm1.xlarge', 'mem': 16384, 'vcpu': 4}, 
        {'id': '6', 'name': 'm1.xxlarge', 'mem': 32768, 'vcpu': 8},
        {'id': '7', 'name': 'm1.xxxlarge', 'mem': 65536, 'vcpu': 16},
    )


# Generated by Django 2.2 on 2019-04-18 07:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0040_auto_20190417_0719'),
    ]

    operations = [
        migrations.AddField(
            model_name='scale',
            name='_remedy_script_scale_in',
            field=models.TextField(blank=True, default='', max_length=5120, verbose_name='scale-in remedy script'),
        ),
        migrations.AlterField(
            model_name='clusteroperation',
            name='script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AlterField(
            model_name='scale',
            name='_remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120, verbose_name='scale-out remedy script'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 00:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0020_auto_20190413_0054'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instanceoperation',
            name='log',
            field=models.TextField(editable=False, max_length=51200, null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-03-30 21:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image',
            name='_remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AlterField(
            model_name='instance',
            name='remedy_script_todo',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
        migrations.AlterField(
            model_name='instanceblueprint',
            name='remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 09:52

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0012_auto_20190406_0938'),
    ]

    operations = [
        migrations.RenameField(
            model_name='image',
            old_name='location',
            new_name='cloud',
        ),
        migrations.RenameField(
            model_name='instance',
            old_name='location',
            new_name='cloud',
        ),
        migrations.RenameField(
            model_name='instanceblueprint',
            old_name='location',
            new_name='cloud',
        ),
        migrations.RenameField(
            model_name='instancetemplate',
            old_name='location',
            new_name='cloud',
        ),
        migrations.RenameField(
            model_name='volume',
            old_name='location',
            new_name='cloud',
        ),
        migrations.AlterField(
            model_name='cloud',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL, verbose_name='admin'),
        ),
        migrations.AlterField(
            model_name='image',
            name='access_id',
            field=models.CharField(max_length=50, verbose_name='actual id on the cloud'),
        ),
        migrations.AlterField(
            model_name='instancetemplate',
            name='access_id',
            field=models.CharField(max_length=50, verbose_name='actual id on the cloud'),
        ),
        migrations.AlterUniqueTogether(
            name='image',
            unique_together={('cloud', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='instanceblueprint',
            unique_together={('cloud', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='instancetemplate',
            unique_together={('cloud', 'name')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-02 21:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='clusteroperation',
            options={'verbose_name': 'operation'},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 06:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0021_auto_20190413_0055'),
        ('engines', '0023_remove_clusteroperation_broadcast'),
    ]

    operations = [
        migrations.CreateModel(
            name='Step',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('remedy_script_todo', models.TextField(blank=True, default='', max_length=5120)),
                ('instances', models.ManyToManyField(blank=True, editable=False, to='clouds.Instance')),
            ],
        ),
        migrations.AddField(
            model_name='cluster',
            name='steps',
            field=models.ManyToManyField(blank=True, editable=False, to='engines.Step'),
        ),
        migrations.AddField(
            model_name='clusteroperation',
            name='steps',
            field=models.ManyToManyField(blank=True, editable=False, to='engines.Step'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-26 02:01

import clouds.base.models
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('engines', '0044_auto_20190424_2135'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='DataEngine',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('_remedy_script', models.TextField(blank=True, default='', max_length=5120)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('type', models.PositiveIntegerField(choices=[(0, 'ralational'), (1, 'raw'), (2, 'freetext'), (3, 'semistructed'), (4, 'event'), (5, 'graph')])),
                ('endpoint', models.CharField(blank=True, max_length=200, null=True)),
                ('description', models.TextField(max_length=5120)),
                ('component', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='engines.Component')),
                ('engine', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='engines.Engine')),
                ('owner', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DataInstance',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50)),
                ('uri_suffix', models.CharField(help_text='the only approach to access this data instance.', max_length=200, verbose_name='suffix of the final uri')),
                ('remedy_script_todo', models.TextField(blank=True, max_length=5120, null=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('public', models.BooleanField(default=False)),
                ('created_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('built_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.PositiveIntegerField(choices=[(0, 'null'), (1, 'active'), (2, 'block'), (3, 'suspend'), (4, 'stop'), (5, 'breakdown'), (6, 'pause'), (7, 'instance_lost')], default=0, editable=False)),
                ('deleting', models.BooleanField(default=False, editable=False)),
                ('cluster', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='engines.Cluster')),
            ],
            bases=(models.Model, clouds.base.models.OperatableMixin),
        ),
        migrations.CreateModel(
            name='DataSource',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('_remedy_script', models.TextField(blank=True, default='', max_length=5120)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('uri', models.CharField(help_text='specify the protocol and address used to sync this remote data source.', max_length=200)),
                ('sync_interval', models.PositiveIntegerField(default=3600, validators=[django.core.validators.MinValueValidator(1)], verbose_name='sync interval in seconds')),
                ('description', models.TextField(max_length=5120)),
                ('owner', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Dataset',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('name', models.CharField(max_length=50, unique=True)),
                ('_remedy_script', models.TextField(blank=True, default='', max_length=5120)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('uri', models.CharField(max_length=200)),
                ('type', models.PositiveIntegerField(blank=True, choices=[(0, 'ralational'), (1, 'raw'), (2, 'freetext'), (3, 'semistructed'), (4, 'event'), (5, 'graph')], null=True)),
                ('size', models.PositiveIntegerField(default=1, validators=[django.core.validators.MinValueValidator(1)])),
                ('description', models.TextField(max_length=5120)),
                ('owner', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
                ('source', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='data.DataSource')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DataInstanceOperation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('batch_uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('script', models.TextField(blank=True, default='', max_length=5120)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('started_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('completed_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.CharField(choices=[('success', 'success'), ('failed', 'failed'), ('running', 'running'), ('waiting', 'waiting')], default='running', editable=False, max_length=50)),
                ('tidied', models.BooleanField(default=False, editable=False)),
                ('manual', models.BooleanField(default=True, editable=False)),
                ('operation', models.CharField(choices=[('start', 'start'), ('stop', 'stop')], default='start', max_length=50)),
                ('log', models.TextField(editable=False, max_length=51200, null=True)),
                ('serial', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='data.DataInstanceOperation')),
                ('target', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='data.DataInstance')),
            ],
            options={
                'verbose_name': 'operation',
                'ordering': ['-started_time', '-completed_time', '-pk'],
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='datainstance',
            name='dataset',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='data.Dataset'),
        ),
        migrations.AddField(
            model_name='datainstance',
            name='engine',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='data.DataEngine'),
        ),
        migrations.AddField(
            model_name='datainstance',
            name='owner',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='datainstance',
            unique_together={('engine', 'uri_suffix'), ('name', 'cluster', 'owner')},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-11 11:53

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0019_auto_20190411_0825'),
    ]

    operations = [
        migrations.AddField(
            model_name='clusteroperation',
            name='serial',
            field=models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='engines.ClusterOperation'),
        ),
    ]



from novaclient import client
from novaclient.exceptions import NotFound
from cinderclient import client as cinder_client
from uuid import uuid4
import time
from django.conf import settings
from ..models import INSTANCE_STATUS

def get_nova(credential):#TODO opt perf.
    return client.Client(credential['api_version'], username=credential['username'], password=credential['password'], project_name=credential['project_name'], auth_url=credential['auth_url'])

def get_cinder(credential):#TODO opt perf.
    return cinder_client.Client(credential['api_version'],credential['username'],credential['password'],credential['project_name'],auth_url="http://controller:35357/v2.0/")

def vm_op(credential, vm_id, op):
    nova=get_nova(credential)
    vm=nova.servers.get(vm_id)
    if op=='start':
        return vm.start()
    elif op=='reboot':
        return vm.reboot()
    elif op=='shutdown' or op=='poweroff':
        return vm.stop()
    else:
        raise Exception('Invalid VM Operation: {}'.format(op))

def vm_vnc_url(credential, vm_id):
    return 'http://# TODO'

def vm_list(credential, include_remarks=[]):
    nova = get_nova(credential)
    return nova.servers.list()

def vm_read(nova, server):
    if not server.name.startswith(settings.PACKONE_LABEL):
        raise Exception('Cannot retrieve non-{} vms!'.format(settings.PACKONE_LABEL))
    flavor=nova.flavors.get(server.flavor['id'])
    return {
        'uuid':server.id,
        'create_time':server.created,
        'ipv4':server.addresses['provider'][0]['addr'],
        'remark':server.name,
        'vcpu':flavor.vcpus,
        'mem':flavor.ram,
    }

def vm_create(credential,image_id,vcpu,mem,template_id=None,remark=''):
    nova=get_nova(credential)
    vm=nova.servers.create(
        name=remark,
        image=image_id,
        flavor=template_id,
        security_groups=[credential['security_group']],
        nics=[{'net-id':credential['net-id']}],
        key_name=credential['key_name']
    )
    mustend = time.time() + 600
    while time.time() < mustend:
        vm=nova.servers.get(vm.id)
        if 'provider' in vm.addresses: break
        time.sleep(5)
    vm.stop()
    return vm_read(nova,vm)

def vm_delete(credential, vm_id):
    nova=get_nova(credential)
    try:
        return nova.servers.delete(vm_id)
    except NotFound as e:
        print(e)

def vm_force_delete(credential, vm_id):
    return vm_delete(credential, vm_id)

def volume_mount(credential, volume_id, vm_id):
    nova=get_nova(credential)
    nova.volumes.create_server_volume(server_id=vm_id,volume_id=volume_id)
    cinder=get_cinder(credential)
    volume=cinder.volumes.get(volume_id)
    mustend = time.time() + 60
    while time.time() < mustend:
        volume=cinder.volumes.get(volume.id)
        if volume.status == 'in-use': break
        time.sleep(1)
    return volume_read(credential,volume.id)

def volume_unmount(credential, volume_id, vm_id):
    nova=get_nova(credential)
    mustend = time.time() + 60
    while time.time() < mustend:
        vm=nova.servers.get(vm_id)
        if vm.status == 'SHUTOFF': break
        time.sleep(1)
    return nova.volumes.delete_server_volume(server_id=vm_id,volume_id=volume_id)

def volume_read(credential,volume_id):
    cinder=get_cinder(credential)
    volume=cinder.volumes.get(volume_id)
    if not volume.name.startswith(settings.PACKONE_LABEL):
        raise Exception('Cannot retrieve non-{} volumes!'.format(settings.PACKONE_LABEL))
    dev=None
    attach_time=None
    if volume.attachments:
        dev=volume.attachments[0]['device']
        attach_time=volume.attachments[0]['attached_at']
    return {
        'remark':volume.name,
        'uuid':volume.id,
        'create_time':volume.created_at,
        'dev':dev,
        'attach_time':attach_time
    }

def volume_create(credential, size, remark=''):
    cinder=get_cinder(credential)
    volume=cinder.volumes.create(
        name=remark,
        size=size
    )
    mustend = time.time() + 60
    while time.time() < mustend:
        volume=cinder.volumes.get(volume.id)
        if volume.status == 'available': break
        time.sleep(1)
    return volume_read(credential,volume.id)

def volume_delete(credential, volume_id):
    cinder=get_cinder(credential)
    mustend = time.time() + 60
    volume=cinder.volumes.get(volume_id)
    while time.time() < mustend:
        if volume.status == 'available': break
        time.sleep(1)
        volume=cinder.volumes.get(volume_id)
    return volume.delete()

def vm_status(credential, vm_id):
    nova=get_nova(credential)
    vm=nova.servers.get(vm_id)
    if vm.status=='ACTIVE':
        return INSTANCE_STATUS.active.value
    elif vm.status=='PAUSED':
        return INSTANCE_STATUS.pause.value
    elif vm.status=='BUILDING':
        return INSTANCE_STATUS.preparing.value
    elif vm.status=='STOPPED':
        return INSTANCE_STATUS.shutdown.value
    elif vm.status=='SHUTOFF':
        return INSTANCE_STATUS.shutdown.value
    elif vm.status=='ERROR':
        return INSTANCE_STATUS.failure.value
    return INSTANCE_STATUS.null.value

def image_list(credential):
    nova = get_nova(credential)
    images=[]
    for image in nova.glance.list():
        images.append({
            'id':image.id,
            'name':image.name,
        })
    return images

def template_list(credential):
    nova = get_nova(credential)
    templates=[]
    for f in nova.flavors.list():
        templates.append({
            'id':f.id,
            'name':f.name,
            'mem':f.ram,
            'vcpu':f.vcpus
        })
    return templates


# Generated by Django 2.2 on 2019-05-30 20:52

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0006_auto_20190528_0258'),
    ]

    operations = [
        migrations.AlterField(
            model_name='datainstance',
            name='cluster',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='engines.Cluster'),
        ),
    ]



# Generated by Django 2.2 on 2019-05-28 04:55

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0054_remove_scale_stack'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cluster',
            name='active',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-11 03:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0015_instanceoperation_tidied'),
        ('engines', '0016_cluster_stepping_instances'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cluster',
            name='stepping_instances',
        ),
        migrations.AddField(
            model_name='cluster',
            name='step_instances',
            field=models.ManyToManyField(blank=True, editable=False, related_name='stepped_clusters', to='clouds.Instance'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-15 01:19

import clouds.base.models
from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import uuid


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('clouds', '0024_instance_remedy_script_todo'),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('remedy_script_todo', models.TextField(blank=True, default='', max_length=51200)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('built_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.PositiveIntegerField(choices=[(0, 'null'), (1, 'active'), (2, 'block'), (3, 'suspend'), (4, 'shutdown'), (5, 'poweroff'), (6, 'breakdown'), (7, 'pause'), (8, 'failure'), (9, 'host_lost'), (10, 'instance_lost'), (11, 'building')], default=11, editable=False)),
                ('deleting', models.BooleanField(default=False, editable=False)),
                ('instances', models.ManyToManyField(blank=True, editable=False, to='clouds.Instance')),
                ('owner', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['pk'],
            },
            bases=(models.Model, clouds.base.models.M2MOperatableMixin),
        ),
        migrations.CreateModel(
            name='GroupOperation',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('batch_uuid', models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False)),
                ('operation', models.CharField(choices=[('start', 'start'), ('reboot', 'reboot'), ('shutdown', 'shutdown'), ('poweroff', 'poweroff'), ('remedy', 'remedy')], default='start', max_length=50)),
                ('script', models.TextField(blank=True, max_length=5120, null=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('started_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('completed_time', models.DateTimeField(blank=True, editable=False, null=True)),
                ('status', models.CharField(choices=[('success', 'success'), ('failed', 'failed'), ('running', 'running'), ('waiting', 'waiting')], default='running', editable=False, max_length=50)),
                ('tidied', models.BooleanField(default=False, editable=False)),
                ('manual', models.BooleanField(default=True, editable=False)),
                ('serial', models.ForeignKey(blank=True, editable=False, null=True, on_delete=django.db.models.deletion.CASCADE, to='clouds.GroupOperation')),
                ('target', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='clouds.Group')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.RemoveField(
            model_name='instanceoperation',
            name='target',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-11 08:25

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0015_instanceoperation_tidied'),
    ]

    operations = [
        migrations.AddField(
            model_name='instanceoperation',
            name='auto',
            field=models.BooleanField(default=True, verbose_name='auto run'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-26 02:36

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='datainstance',
            name='public',
        ),
        migrations.AlterField(
            model_name='datainstance',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='datainstance',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='datainstance',
            name='remedy_script_todo',
            field=models.TextField(blank=True, default='', max_length=51200),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 22:10

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0007_auto_20190406_2130'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='cluster',
            name='scale',
        ),
        migrations.AlterField(
            model_name='cluster',
            name='current_step',
            field=models.PositiveIntegerField(default=0, editable=False, validators=[django.core.validators.MinValueValidator(0)]),
        ),
    ]



# Generated by Django 2.2 on 2019-04-28 00:21

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0038_auto_20190427_0841'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('engines', '0044_auto_20190424_2135'),
    ]

    operations = [
        migrations.CreateModel(
            name='Stack',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=50, unique=True)),
                ('_remedy_script', models.TextField(blank=True, default='', max_length=5120)),
                ('public', models.BooleanField(default=False)),
                ('enabled', models.BooleanField(default=True)),
                ('modified_time', models.DateTimeField(auto_now=True)),
                ('created_time', models.DateTimeField(auto_now_add=True)),
                ('remark', models.CharField(blank=True, max_length=100, null=True)),
                ('_driver', models.CharField(max_length=50)),
                ('host', models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Instance')),
                ('owner', models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 21:30

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0006_auto_20190406_2130'),
    ]

    operations = [
        migrations.AlterField(
            model_name='scale',
            name='step_blueprints',
            field=models.ManyToManyField(blank=True, related_name='stepped_scales', to='clouds.InstanceBlueprint'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 09:38

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):
    dependencies = [
        ('clouds', '0011_auto_20190404_2347'),
    ]

    operations = [
	migrations.RenameModel('Location', 'Cloud'),
        migrations.AlterField(
            model_name='image',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Cloud'),
        ),
        migrations.AlterField(
            model_name='instance',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Cloud'),
        ),
        migrations.AlterField(
            model_name='instanceblueprint',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Cloud'),
        ),
        migrations.AlterField(
            model_name='instancetemplate',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Cloud'),
        ),
        migrations.AlterField(
            model_name='volume',
            name='location',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='clouds.Cloud'),
        ),
    ]



from ambari.client import Client

def get_engine_host(portal, engine):
    c=Client(portal)
    return c.cluster.hosts[-1].name

def list_engines(portal):
    c=Client(portal)
    engines=[]
    for s in c.stack_services():
        engines.append({
            'name':s['StackServices']['service_name'],
            'description':s['StackServices']['comments']
        })
    return engines

def list_components(portal,engine):
    c=Client(portal)
    components=[]
    for cpn in c.stack_service_components(service_name=engine):
        components.append({
            'name':cpn['StackServiceComponents']['component_name'],
            'type':cpn['StackServiceComponents']['component_category'].lower()
        })
    return components


# Generated by Django 2.1.7 on 2019-04-03 20:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0005_auto_20190402_2147'),
    ]

    operations = [
        migrations.RenameField(
            model_name='location',
            old_name='platform',
            new_name='_driver',
        ),
        migrations.AlterField(
            model_name='instancetemplate',
            name='remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-04 01:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0007_auto_20190404_0118'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instanceblueprint',
            name='name',
            field=models.CharField(max_length=50),
        ),
    ]



# Generated by Django 2.2 on 2019-05-23 05:03

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0051_remove_cluster_active'),
    ]

    operations = [
        migrations.AddField(
            model_name='cluster',
            name='active',
            field=models.BooleanField(default=True, editable=False),
        ),
    ]



# Generated by Django 2.2 on 2019-05-28 02:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0053_auto_20190528_0258'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='scale',
            name='stack',
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-04 23:46

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0009_auto_20190404_0151'),
    ]

    operations = [
        migrations.AlterField(
            model_name='volume',
            name='status',
            field=models.PositiveIntegerField(choices=[(0, 'null'), (1, 'active'), (2, 'block'), (3, 'suspend'), (4, 'shutdown'), (5, 'poweroff'), (6, 'breakdown'), (7, 'pause'), (8, 'failure'), (9, 'host_lost'), (10, 'instance_lost'), (11, 'building')], default=11, editable=False),
        ),
        migrations.AlterUniqueTogether(
            name='instanceblueprint',
            unique_together={('location', 'name')},
        ),
        migrations.AlterUniqueTogether(
            name='instancetemplate',
            unique_together={('location', 'name')},
        ),
    ]



from setuptools import setup, find_packages

setup(
    name = "pk1",
    version = "0.0.6",
    keywords = ("pip", "packone"),
    description = "PackOne: Pack clouds, engines and data services into one light stack.",
    long_description = open('README.rst').read(),
    license = "Apache-2.0 Licence",
    url = "https://github.com/cas-bigdatalab/packone",
    author = "Excel Wang",
    author_email = "wanghj@cnic.com",
    packages = find_packages(),
    include_package_data = True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires = ['uwsgi','pk1-remedy','django-admin-auto','djangorestframework','djangorestframework-simplejwt', 'django-filter', 'psycopg2-binary', 'coreapi', 'paramiko', 'scp', 'django-cors-headers', 'python-novaclient', 'python-cinderclient', 'django-autocomplete-light'],
    entry_points = {
        'console_scripts': [
            'pk1 = pk1.server:main',
        ]
    }
)


# Generated by Django 2.1.7 on 2019-04-10 02:59

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0013_auto_20190406_0952'),
    ]

    operations = [
        migrations.AddField(
            model_name='cloud',
            name='hosts',
            field=models.TextField(blank=True, max_length=5120, null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 21:26

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0004_auto_20190405_0329'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='scale',
            name='uuid',
        ),
    ]



# Generated by Django 2.2 on 2019-04-24 21:35

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0043_auto_20190420_0247'),
    ]

    operations = [
        migrations.AddField(
            model_name='scale',
            name='_remedy_script_scale_out',
            field=models.TextField(blank=True, default='', max_length=5120, verbose_name='scale-out remedy script'),
        ),
        migrations.AlterField(
            model_name='scale',
            name='_remedy_script',
            field=models.TextField(blank=True, default='', max_length=5120, verbose_name='initial remedy script'),
        ),
        migrations.AlterField(
            model_name='scale',
            name='init_blueprints',
            field=models.ManyToManyField(related_name='initialized_scales', to='clouds.InstanceBlueprint', verbose_name='initial blueprints'),
        ),
        migrations.AlterField(
            model_name='scale',
            name='step_blueprints',
            field=models.ManyToManyField(blank=True, related_name='stepped_scales', to='clouds.InstanceBlueprint', verbose_name='scale-out blueprints'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 11:11

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0021_auto_20190413_0055'),
    ]

    operations = [
        migrations.AlterField(
            model_name='instanceoperation',
            name='batch_uuid',
            field=models.UUIDField(auto_created=True, default=uuid.uuid4, editable=False),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-09 01:16

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0011_cluster_deleting'),
    ]

    operations = [
        migrations.AddField(
            model_name='clusteroperation',
            name='script',
            field=models.TextField(blank=True, max_length=5120, null=True),
        ),
    ]



# Generated by Django 2.2 on 2019-06-17 02:15

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('data', '0010_auto_20190614_2013'),
    ]

    operations = [
        migrations.AlterField(
            model_name='dataengine',
            name='type',
            field=models.PositiveIntegerField(choices=[(0, 'ralational'), (1, 'raw'), (2, 'freetext'), (3, 'semistructed'), (4, 'event'), (5, 'graph'), (6, 'star_catalogue')]),
        ),
        migrations.AlterField(
            model_name='dataset',
            name='type',
            field=models.PositiveIntegerField(blank=True, choices=[(0, 'ralational'), (1, 'raw'), (2, 'freetext'), (3, 'semistructed'), (4, 'event'), (5, 'graph'), (6, 'star_catalogue')], null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-10 20:50

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0013_remove_scale_hosts'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cluster',
            name='engines',
            field=models.ManyToManyField(blank=True, to='engines.Engine'),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-02 02:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0003_instanceoperation_started_time'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='instanceblueprint',
            options={'verbose_name': 'blueprint'},
        ),
        migrations.AlterModelOptions(
            name='instanceoperation',
            options={'verbose_name': 'operation'},
        ),
        migrations.RemoveField(
            model_name='instance',
            name='monitoring',
        ),
        migrations.RemoveField(
            model_name='mount',
            name='deleting',
        ),
        migrations.AddField(
            model_name='instance',
            name='built_time',
            field=models.DateTimeField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='mount',
            name='built_time',
            field=models.DateTimeField(blank=True, editable=False, null=True),
        ),
        migrations.AddField(
            model_name='volume',
            name='built_time',
            field=models.DateTimeField(blank=True, editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='instance',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='instanceoperation',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='mount',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
        migrations.AlterField(
            model_name='volume',
            name='created_time',
            field=models.DateTimeField(auto_now_add=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-05 02:53

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('user', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='profile',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL),
        ),
    ]



# Generated by Django 2.2 on 2019-04-17 07:19

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0039_auto_20190417_0614'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='stepoperation',
            options={'verbose_name': 'operation'},
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-13 09:21

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0029_auto_20190413_0836'),
    ]

    operations = [
        migrations.AddField(
            model_name='step',
            name='deleting',
            field=models.BooleanField(default=False, editable=False),
        ),
        migrations.AlterField(
            model_name='cluster',
            name='built_time',
            field=models.DateTimeField(blank=True, default=django.utils.timezone.now, editable=False, null=True),
        ),
        migrations.AlterField(
            model_name='step',
            name='built_time',
            field=models.DateTimeField(blank=True, default=django.utils.timezone.now, editable=False, null=True),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-11 01:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0015_instanceoperation_tidied'),
        ('engines', '0015_auto_20190410_2220'),
    ]

    operations = [
        migrations.AddField(
            model_name='cluster',
            name='stepping_instances',
            field=models.ManyToManyField(blank=True, editable=False, related_name='stepping_clusters', to='clouds.Instance'),
        ),
    ]



# Generated by Django 2.2 on 2019-04-17 07:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('clouds', '0030_auto_20190417_0157'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='hosts',
            field=models.TextField(default='###group {}###\n', editable=False, max_length=5120),
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 22:11

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0008_auto_20190406_2210'),
    ]

    operations = [
        migrations.AddField(
            model_name='cluster',
            name='scale',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.PROTECT, to='engines.Scale'),
            preserve_default=False,
        ),
    ]



# Generated by Django 2.1.7 on 2019-04-06 22:45

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('engines', '0009_cluster_scale'),
    ]

    operations = [
        migrations.AlterField(
            model_name='cluster',
            name='instances',
            field=models.ManyToManyField(blank=True, editable=False, to='clouds.Instance'),
        ),
        migrations.AlterField(
            model_name='cluster',
            name='owner',
            field=models.ForeignKey(editable=False, on_delete=django.db.models.deletion.PROTECT, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterField(
            model_name='cluster',
            name='remark',
            field=models.CharField(blank=True, max_length=100, null=True),
        ),
    ]
